#ifndef __NQDS_THREAD_INCLUDED
#define __NQDS_THREAD_INCLUDED
#include "NQDSProto.h"

#define PACKETS_LOST_TOLERANCE 1000

class LostPacketsLog
{
public:
	LostPacketsLog();

	BOOL AddToLog( DWORD aSeq );
	BOOL DeleteFromLog( DWORD aSeq );
	void Reset();

	DWORD m_Items[PACKETS_LOST_TOLERANCE];
	int m_nFirst;
	int m_nLast;
};

#define BROADCAST_QUEUE_SIZE 100

class CBroadcastThread
{
public:
	CBroadcastThread();
	~CBroadcastThread();
	BOOL CreateThread( in_addr addrSelf, WORD nPort );
	virtual void InitInstance();
	virtual void ExitInstance();

	int PreparePacket();
	QUOT_BROADCAST *PacketPointer( int nIndex ){ return (-1==nIndex)?NULL:&m_PacketsQueue[nIndex]; };
	void EndPacket( int nIndex );

	void SignalTerminate(){ m_bTerminateSignal = TRUE; };
	void SafeTerminateAndCleanUp();

	BOOL IsAlive();
	BOOL IsValidThread(){return (BOOL)(m_hThread);};
private:
	// Thread data
	HANDLE m_hThread;
	DWORD m_nThreadID;
	BOOL m_bTerminateSignal;
	BOOL m_bTerminated;
	// Socket data
	SOCKET m_hSocket;
	WORD m_nPort;
	HANDLE m_hIOEvent;
	// Statistics
	DWORD m_nPacketsSum;
	DWORD m_nPacketsLost;
	DWORD m_nLastSeqNum;
	LostPacketsLog m_LostPackets;
	// Broadcast
	QUOT_BROADCAST m_PacketsQueue[BROADCAST_QUEUE_SIZE];
	CRITICAL_SECTION m_QueueLocks[BROADCAST_QUEUE_SIZE];
	CRITICAL_SECTION m_hCriticalSection;
	HANDLE m_hSemaphore;
	int m_nQueueHead;
	int m_nQueueTail;
};

class CNQDSThread
{
public:
	static CBroadcastThread m_BroadcastThread;
	enum SERVER_STATE
	{
		ST_IDLE,
		ST_READY,
		ST_HALT
	};
	CNQDSThread();
	~CNQDSThread();
	BOOL CreateThread( in_addr addrSelf, in_addr addrPeer, WORD nPort );

	virtual void InitInstance();
	virtual void ExitInstance();

	void ProcessData( char *buf, int nLen );

	void SignalTerminate(){ m_bTerminateSignal = TRUE; };
	void SafeTerminateAndCleanUp();
	BOOL IsAlive();
	BOOL IsValidThread(){return (BOOL)(m_hThread);};

	// Log lost packets
	void LogLostPacket( DWORD i ){m_LostPackets.AddToLog( i );m_nPacketsLost++;};
	DWORD GetLostCount(){ return m_nPacketsLost; };
private:
	// Thread data
	HANDLE m_hThread;
	DWORD m_nThreadID;
	BOOL m_bTerminateSignal;
	BOOL m_bTerminated;
	// Socket data
	SOCKET m_hSocket;
	HANDLE m_hIOEvent;
	// Statistics
	DWORD m_nPacketsSum;
	DWORD m_nPacketsLost;
	DWORD m_nLastSeqNum;
	LostPacketsLog m_LostPackets;
	// State
	SERVER_STATE m_Status;
};

#endif //__NQDS_THREAD_INCLUDED
